
template <int order, typename coef, int step>
void cos_int_dom<order,coef,step>::divide_row (
    OAC  & numer_row,
    coef & numer_sca,
    coef const & denom
) {
    for (int p {step}; p < order; p += step)
        numer_row.at(p) /= denom;
    numer_sca /= denom;
}

template <int order, typename coef, int step>
void cos_int_dom<order,coef,step>::reduce_row (
    OAC  & row,
    coef & sca
) {
    coef fac (grt_com_fac (row.at(1 * step), row.at(2 * step)));
    for (int p {3 * step}; p < order; p += step)
        fac = grt_com_fac (fac, row.at(p));
    fac = grt_com_fac (fac, sca);
    if (fac < 2) return;
    divide_row (row, sca, fac);
}

template <int order, typename coef, int step>
void cos_int_dom<order,coef,step>::sub_row (
    OAC  const & subt_row,
    OAC        & minu_row,
    coef const & subt_x,
    coef       & minu_y,
    int  const   col
) {
    coef mrac (minu_row.at(col));
    coef srac (subt_row.at(col));
    coef const gcf (grt_com_fac (mrac, srac));
    if (gcf > 1) {
        mrac /= gcf;
        srac /= gcf;
    }
    for (int p {step}; p < order; p += step) {
        coef & mrp {minu_row.at(p)};
        mrp = mrp * srac - subt_row.at(p) * mrac;
    }
    minu_y = minu_y * srac - subt_x * mrac;
    reduce_row (minu_row, minu_y);
}

template <int order, typename coef, int step>
void cos_int_dom<order,coef,step>::view_system (
    OAOAC const & sys,
    OAC   const & sca
) {
    cout << "\n view_system ";

    for (int p {step}; p < order; p += step) {
        OAC const & sys_p {sys.at(p)};
        cout << "\n ";
        for (int q {step}; q < order; q += step) {
            coef const & sys_p_q {sys_p.at(q)};
            if (sys_p_q == 0)
                cout << noshowpos;
            else
                cout << showpos;
            cout << " " << setw (4) << sys_p_q;
        }

        coef const & sys_p_p {sca.at(p)};
        if (sys_p_p == 0)
            cout << noshowpos;
        else
            cout << showpos;
        cout << "   $ " << setw (4) << sys_p_p;
    }
    cout << noshowpos;
}

template <int order, typename coef, int step>
void cos_int_dom<order,coef,step>::non_integral (
    OAOAC const & sys,
    OAC   const & sca
) {
    cout << "\n\n cos_int_dom ... non-integral quotient: ";

    double total {0.0};

    for (int p {step}; p < order; p += step) {
        OAC const & sys_p {sys.at(p)};
        cout << "\n ";
        int perma_q {-1};
        for (int q {step}; q < order; q += step) {
            coef const & sys_p_q {sys_p.at(q)};
            if (sys_p_q == 0) continue;

            if (perma_q != -1) throw fail
                {"cos_int_dom::non_integral ... perma_q != -1"};
            perma_q = q;

            cout << "C" << q << "/" << order;
        }

        coef const & sys_p_p {sca.at(p)};
        if (sys_p_p == 0)
            cout << noshowpos;
        else
            cout << showpos;
        cout << " = " << sys_p_p;

        cout << " / " ;
        cout << showpos << sys_p.at(perma_q) << noshowpos;

        double const approx {double (sys_p_p) / double (sys_p.at(perma_q))};
        total += approx * two_cos (perma_q);

        cout << " approx " << approx;

    }
    cout << "\n real value: " << total;
    cout << noshowpos;
}

template <int order, typename coef, int step>
cos_int_dom<order,coef,step>
cos_int_dom<order,coef,step>::operator/ (cos_int_dom const & o) const {
    OAOAC sys;
    OAC sca;

    {
        for (int p {step}; p < order; p += step) {
            cos_int_dom const addend (o.at(p) * 2);
            for (int q {step}; q < order; q += step)
                sys.at(q).at(p) += addend.at(q);
        }
    }

    for (int p {step}; p < order; p += step) {
        for (int q {step}; q < order; q += step) {
            {
                int const sum {p + q};
                if (sum < order)
                    sys.at(sum).at(p) += o.at(q);
                else if (sum > order)
                    sys.at(2 * order - sum).at(p) -= o.at(q);
            } {
                int const dif {p - q};
                if (dif > 0)
                    sys.at(+ dif).at(p) += o.at(q);
                else if (dif < 0)
                    sys.at(- dif).at(p) += o.at(q);
            }
        }
    }

    for (int p {step}; p < order; p += step) {
        sca.at(p) = at(p);
        reduce_row (sys.at(p), sca.at(p));
    }

    {
        offset_array<order,bool,step> row_done {false};
        for (int c {step}; c < order; c += step) {
            int r (step);
            for ( ; r < order; r += step) {
                if (row_done.at(r)) continue;
                if (sys.at(r).at(c) == 0) continue;
                break;
            }
            if (r >= order) {
                throw fail ("cos_int_dom::operator/ ... division by zero");
            }

            for (int q {step}; q < order; q += step) {
                if (q == r) continue;
                sub_row (sys.at(r), sys.at(q),
                    sca.at(r), sca.at(q), c);
            }

            row_done.at(r) = true;
        }        
    } 

    {
        cos_int_dom quo (0);

        offset_array<order,bool,step> row_done {false};

        for (int c {step}; c < order; c += step) {
            int r {step};
            for ( ; r < order; r += step) {
                if (row_done.at(r)) continue;
                if (sys.at(r).at(c) == 0) continue;
                break;
            }
            if (r >= order) {
                throw fail ("cos_int_dom::operator/ ... division by zero");
            }

            OAC const & sr {sys.at(r)};

            if (sr.at(c) == +1) {
                 quo.at(c) += sca.at(r);
            } else if (sr.at(c) == -1) {
                 quo.at(c) -= sca.at(r);
            } else {
                non_integral (sys, sca);
                throw fail ("cos_int_dom::operator/ ...quotient not an integer");
            }
            row_done.at(r) = true;
        }    

        return quo;
    }

}

