
template <int order, typename coef, int step>
void cos_int_dom<order,coef,step>::html (
    ostream & o,
    cos_int_dom<order,coef,step> const & t,
    bool anything
) {
    static char const * const C {"<b>C</b><sub>"};
    static char const * const D {"/"};
    static char const * const E {"</sub>"};

    bool const show_order {true};

    for (int p {step}; p < order; p += step) {
        coef const & c {t.at(p)};

        if (c == +1) {
            if (anything) o << " + ";
            o << C << p;
            if (show_order) o << D << order;
            o << E;
            anything = true;
        } else if (c == -1) {
            if (anything) o << " ";
            o << "&minus; " << C << p;
            if (show_order) o << D << order;
            o << E;
            anything = true;
        } else if (c > 0) {
            if (anything) o << " + ";
            o << +c << C << p;
            if (show_order) o << D << order;
            o << E;
            anything = true;
        } else if (c < 0) {
            if (anything) o << " ";
            o << "&minus; " << -c << C << p;
            if (show_order) o << D << order;
            o << E;
            anything = true;
        }
    }

    if (! anything) cout << " zero ";
}

template <int order, typename coef, int step>
void cos_int_dom<order,coef,step>::mul_table () {
    static char const * const yel {" bgcolor=\"F6F6BB\""};
    static char const * const grn {" bgcolor=\"BBEEBB\""};
    static char const * const red {" bgcolor=\"FFCCCC\""};
    static char const * const blu {" bgcolor=\"CCCCFF\""};
    static char const * const pur {" bgcolor=\"E5CCE5\""};
    static char const * const drk {" bgcolor=\"333333\""};

    static constexpr int show_double {true};

    using kind = cos_int_dom<order,big_int,step>;
    static kind const deuce (2);

    ofstream o ("/Users/davebarber/Desktop/web_support"
                "/maffus/algebra/cos_int_dom/temp.html");

    o.setf (std::ios::fixed, std::ios::floatfield);
    o.precision (dbl_prec);

    o << "\n\n<p align=center>";
    o << "<table cellpadding=5" << drk << ">";

    {
        o << "\n\n<tr align=center>";
        o << "<td" << yel << "><b>multiplication</b><br>order " << order;
        for (int c {step}; c < order; c += step) {
            kind const tc {kind::single (c)};
            o << "\n<td" << grn << ">";
            html (o, tc);
            if (show_double) o << "<br>" << double (tc);
        }
    }

    for (int r {step}; r < order; r += step) {
        kind const tr {kind::single (r)};
        o << "\n\n<tr align=center>";

        o << "\n<td" << grn << ">";
        html (o, tr);
        if (show_double) o << "<br>" << double (tr);

        for (int c {step}; c < order; c += step) {
            kind const tc {kind::single (c)};

            kind const trc {tr * tc};

            o << "\n<td";
            if ((r + c) == order) {
                if (r == c) o << pur;
                       else o << blu;
            } else {
                if (r == c) o << red;
                       else o << yel;
            }
            o << ">";

            if (r != c) {
                html (o, trc);
            } else {
                kind const trc_2 {trc - deuce};
                o << "2";
                html (o, trc_2, true);
            }

            if (show_double) o << "<br>" << double (trc);
        }
    }

    o << "\n\n<tr align=center>";
    o << "<td" << yel << ">&nbsp;";
    o << "\n<td colspan=" << ((order - 1) / step) << grn << ">";
    o << "2 = "; html (o, deuce);
    o << "\n\n</table>";
}

