
template <int a_order, int b_order, typename coef, int step>
cos_int_dom<lst_com_mul(a_order,b_order),coef,step> operator+ (
    cos_int_dom<a_order,coef,step> const & a,
    cos_int_dom<b_order,coef,step> const & b
) {
    static_assert (a_order != b_order, "wrong operator+ for a_order == b_order");

    cos_int_dom<lst_com_mul(a_order,b_order),coef,step> x (a);
    cos_int_dom<lst_com_mul(a_order,b_order),coef,step> y (b);

    cos_int_dom<
        lst_com_mul(a_order,b_order),
        typename std::enable_if <(a_order != b_order),coef>::type,
        step
    > const z {x.operator+ (y)};

    return z;
}

template <int a_order, int b_order, typename coef, int step>
cos_int_dom<lst_com_mul(a_order,b_order),coef,step> operator- (
    cos_int_dom<a_order,coef,step> const & a,
    cos_int_dom<b_order,coef,step> const & b
) {
    static_assert (a_order != b_order, "wrong operator- for a_order == b_order");

    cos_int_dom<lst_com_mul(a_order,b_order),coef,step> x (a);
    cos_int_dom<lst_com_mul(a_order,b_order),coef,step> y (b);

    cos_int_dom<
        lst_com_mul(a_order,b_order),
        typename std::enable_if <(a_order != b_order),coef>::type,
        step
    > const z {x.operator- (y)};

    return z;
}

template <int a_order, int b_order, typename coef, int step>
cos_int_dom<lst_com_mul(a_order,b_order),coef,step> operator* (
    cos_int_dom<a_order,coef,step> const & a,
    cos_int_dom<b_order,coef,step> const & b
) {
    static_assert (a_order != b_order, "wrong operator* for a_order == b_order");

    cos_int_dom<lst_com_mul(a_order,b_order),coef,step> x (a);
    cos_int_dom<lst_com_mul(a_order,b_order),coef,step> y (b);

    cos_int_dom<
        lst_com_mul(a_order,b_order),
        typename std::enable_if <(a_order != b_order),coef>::type,
        step
    > const z {x.operator* (y)};

    return z;
}

template <int a_order, int b_order, typename coef, int step>
cos_int_dom<lst_com_mul(a_order,b_order),coef,step> operator/ (
    cos_int_dom<a_order,coef,step> const & a,
    cos_int_dom<b_order,coef,step> const & b
) {
    static_assert (a_order != b_order, "wrong operator/ for a_order == b_order");

    cos_int_dom<lst_com_mul(a_order,b_order),coef,step> x (a);
    cos_int_dom<lst_com_mul(a_order,b_order),coef,step> y (b);

    cos_int_dom<
        lst_com_mul(a_order,b_order),
        typename std::enable_if <(a_order != b_order),coef>::type,
        step
    > const z {x.operator/ (y)};

    return z;
}

//////////////////////////

template <int order, typename coef>
cos_int_dom<order,coef,1> operator+ (
    cos_int_dom<order,coef,1> const & a,
    cos_int_dom<order,coef,2> const & b
) {
    return a + cos_int_dom<order,coef,1> {b};
}

template <int order, typename coef>
cos_int_dom<order,coef,1> operator+ (
    cos_int_dom<order,coef,2> const & a,
    cos_int_dom<order,coef,1> const & b
) {
    return cos_int_dom<order,coef,1> {a} + b;
}

template <int order, typename coef>
cos_int_dom<order,coef,1> operator- (
    cos_int_dom<order,coef,1> const & a,
    cos_int_dom<order,coef,2> const & b
) {
    return a - cos_int_dom<order,coef,1> {b};
}

template <int order, typename coef>
cos_int_dom<order,coef,1> operator- (
    cos_int_dom<order,coef,2> const & a,
    cos_int_dom<order,coef,1> const & b
) {
    return cos_int_dom<order,coef,1> {a} - b;
}

template <int order, typename coef>
cos_int_dom<order,coef,1> operator* (
    cos_int_dom<order,coef,1> const & a,
    cos_int_dom<order,coef,2> const & b
) {
    return a * cos_int_dom<order,coef,1> {b};
}

template <int order, typename coef>
cos_int_dom<order,coef,1> operator* (
    cos_int_dom<order,coef,2> const & a,
    cos_int_dom<order,coef,1> const & b
) {
    return cos_int_dom<order,coef,1> {a} * b;
}

template <int order, typename coef>
cos_int_dom<order,coef,1> operator/ (
    cos_int_dom<order,coef,1> const & a,
    cos_int_dom<order,coef,2> const & b
) {
    return a / cos_int_dom<order,coef,1> {b};
}

template <int order, typename coef>
cos_int_dom<order,coef,1> operator/ (
    cos_int_dom<order,coef,2> const & a,
    cos_int_dom<order,coef,1> const & b
) {
    return cos_int_dom<order,coef,1> {a} / b;
}




