
struct fail {
   fail (char const * const m) 
      { cout << "\n\n *** fail *** " << m << " *** \n\n"; }   
};

template <int order, typename compo, int step>
class offset_array {
    static_assert (step >= 1, "offset_array ... step < 1");
    static_assert (step <= 2, "offset_array ... step > 2");
public:
    static constexpr int size {
        step == 1 ? (order - 1) / step :
        order % 2 ? (order - 1) / step :
                    (order - 2) / step
    };
private:
    compo guts[size];
public:
    offset_array (offset_array const &) = default;
    offset_array (offset_array &&) = default;
    offset_array & operator= (offset_array const &) = default;
    offset_array & operator= (offset_array &&) = default;
    ~offset_array () = default;

    explicit offset_array (compo const & value = compo {}) {
        for (int n {0}; n < size; ++n)
            guts[n] = value;
    }

    compo & at (int const n) {
        if (n <    1  ) throw fail {"offset_array::at ... n <    1  "};
        if (n >= order) throw fail {"offset_array::at ... n >= order"};
        if (n %  step ) throw fail {"offset_array::at ... n %  step "};
        return guts[n / step - 1];
    }

    compo const & at (int const n) const {
        if (n <    1  ) throw fail {"offset_array::at ... n <    1  "};
        if (n >= order) throw fail {"offset_array::at ... n >= order"};
        if (n %  step ) throw fail {"offset_array::at ... n %  step "};
        return guts[n / step - 1];
    }
};

template <int order, typename compo, int step>
bool isomeric (
    offset_array<order,compo,step> const & a,
    offset_array<order,compo,step> const & b
) {
    for (int p {step}; p < order; p += step)
        if (a.at(p) != b.at(p)) return false;
    return true;
}

template <int order, typename compo, int step>
bool anisomeric (
    offset_array<order,compo,step> const & a,
    offset_array<order,compo,step> const & b
) {
    for (int p {step}; p < order; p += step)
        if (a.at(p) != b.at(p)) return true;
    return false;
}

template <typename coef>
constexpr coef grt_com_fac (coef x, coef y) {
    return x <  0 ? grt_com_fac (-x, +y)
         : y <  0 ? grt_com_fac (+x, -y)
         : x == 0 ? y
         : y == 0 ? x
         : grt_com_fac (y % x, x);
}

template <typename coef>
constexpr coef lst_com_mul (coef x, coef y) {
    return x <  0 ? lst_com_mul (-x, +y)
         : y <  0 ? lst_com_mul (+x, -y)
         : x == 0 ? y
         : y == 0 ? x
         : (x * y) / grt_com_fac (x, y);
}
